@@ -338,12 +338,9 @@ struct PyHandler {
 
             Py_DECREF(pystr);
         } else {
-            char zstr[length + 1];
+            std::string zstr(str, length);
 
-            strncpy(zstr, str, length);
-            zstr[length] = '\0';
-
-            value = PyLong_FromString(zstr, NULL, 10);
+            value = PyLong_FromString(zstr.c_str(), NULL, 10);
         }
 
         if (value == NULL) {
@@ -745,6 +742,7 @@ rapidjson_loads(PyObject* self, PyObject* args, PyObject* kwargs)
                                      &allowNan,
                                      &datetimeModeObj,
                                      &uuidModeObj))
+        return NULL;
 
     if (objectHook && !PyCallable_Check(objectHook)) {
         PyErr_SetString(PyExc_TypeError, "object_hook is not callable");
@@ -769,18 +767,30 @@ rapidjson_loads(PyObject* self, PyObject* args, PyObject* kwargs)
         return NULL;
     }
 
-    if (datetimeModeObj && PyLong_Check(datetimeModeObj)) {
-        datetimeMode = (DatetimeMode) PyLong_AsLong(datetimeModeObj);
-        if (datetimeMode < DATETIME_MODE_NONE || datetimeMode > DATETIME_MODE_ISO8601_UTC) {
-            PyErr_SetString(PyExc_ValueError, "Invalid date_time");
+    if (datetimeModeObj) {
+        if (PyLong_Check(datetimeModeObj)) {
+            datetimeMode = (DatetimeMode) PyLong_AsLong(datetimeModeObj);
+            if (datetimeMode < DATETIME_MODE_NONE || datetimeMode > DATETIME_MODE_ISO8601_UTC) {
+                PyErr_SetString(PyExc_ValueError, "Invalid datetime_mode");
+                return NULL;
+            }
+        }
+        else {
+            PyErr_SetString(PyExc_TypeError, "datetime_mode must be an integer value");
             return NULL;
         }
     }
 
-    if (uuidModeObj && PyLong_Check(uuidModeObj)) {
-        uuidMode = (UuidMode) PyLong_AsLong(uuidModeObj);
-        if (uuidMode < UUID_MODE_NONE || uuidMode > UUID_MODE_HEX) {
-            PyErr_SetString(PyExc_ValueError, "Invalid uuid_time");
+    if (uuidModeObj) {
+        if (PyLong_Check(uuidModeObj)) {
+            uuidMode = (UuidMode) PyLong_AsLong(uuidModeObj);
+            if (uuidMode < UUID_MODE_NONE || uuidMode > UUID_MODE_HEX) {
+                PyErr_SetString(PyExc_ValueError, "Invalid uuid_mode");
+                return NULL;
+            }
+        }
+        else {
+            PyErr_SetString(PyExc_TypeError, "uuid_mode must be an integer value");
             return NULL;
         }
     }
@@ -831,22 +841,24 @@ rapidjson_loads(PyObject* self, PyObject* args, PyObject* kwargs)
 
 struct WriterContext {
     const char* key;
+    Py_ssize_t size;
     PyObject* object;
     PyObject* decref;
     unsigned level;
     bool isObject;
 
-    WriterContext(const char* k, PyObject* o, bool isO, int l, PyObject* d=NULL)
-    : key(k), object(o), decref(d), level(l), isObject(isO)
+    WriterContext(const char* k, Py_ssize_t s, PyObject* o, bool isO, int l, PyObject* d=NULL)
+    : key(k), size(s), object(o), decref(d), level(l), isObject(isO)
     {}
 };
 
 struct DictItem {
     char* key_str;
+    Py_ssize_t key_size;
     PyObject* item;
 
-    DictItem(char* k, PyObject* i)
-    : key_str(k), item(i)
+    DictItem(char* k, Py_ssize_t s, PyObject* i)
+    : key_str(k), key_size(s), item(i)
     {}
 
     bool operator<(const DictItem& other) const {
@@ -876,7 +888,7 @@ rapidjson_dumps_internal(
     std::vector<WriterContext> stack;
     stack.reserve(128);
 
-    stack.push_back(WriterContext(NULL, value, false, 0));
+    stack.push_back(WriterContext(NULL, 0, value, false, 0));
 
     while (!stack.empty()) {
         const WriterContext& current = stack.back();
@@ -893,7 +905,7 @@ rapidjson_dumps_internal(
 
         if (object == NULL) {
             if (current.key != NULL) {
-                writer->Key(current.key);
+                writer->Key(current.key, current.size);
             }
             else if (current.decref != NULL) {
                 Py_DECREF(current.decref);
@@ -985,29 +997,29 @@ rapidjson_dumps_internal(
         }
         else if (PyList_Check(object)) {
             writer->StartArray();
-            stack.push_back(WriterContext(NULL, NULL, false, currentLevel));
+            stack.push_back(WriterContext(NULL, 0, NULL, false, currentLevel));
 
             Py_ssize_t size = PyList_GET_SIZE(object);
 
             for (Py_ssize_t i = size - 1; i >= 0; --i) {
                 PyObject* item = PyList_GET_ITEM(object, i);
-                stack.push_back(WriterContext(NULL, item, false, nextLevel));
+                stack.push_back(WriterContext(NULL, 0, item, false, nextLevel));
             }
         }
         else if (PyTuple_Check(object)) {
             writer->StartArray();
-            stack.push_back(WriterContext(NULL, NULL, false, currentLevel));
+            stack.push_back(WriterContext(NULL, 0, NULL, false, currentLevel));
 
             Py_ssize_t size = PyTuple_GET_SIZE(object);
 
             for (Py_ssize_t i = size - 1; i >= 0; --i) {
                 PyObject* item = PyTuple_GET_ITEM(object, i);
-                stack.push_back(WriterContext(NULL, item, false, nextLevel));
+                stack.push_back(WriterContext(NULL, 0, item, false, nextLevel));
             }
         }
         else if (PyDict_Check(object)) {
             writer->StartObject();
-            stack.push_back(WriterContext(NULL, NULL, true, currentLevel));
+            stack.push_back(WriterContext(NULL, 0, NULL, true, currentLevel));
 
             Py_ssize_t pos = 0;
             PyObject* key;
@@ -1016,9 +1028,10 @@ rapidjson_dumps_internal(
             if (!sortKeys) {
                 while (PyDict_Next(object, &pos, &key, &item)) {
                     if (PyUnicode_Check(key)) {
-                        char* key_str = PyUnicode_AsUTF8(key);
-                        stack.push_back(WriterContext(NULL, item, false, nextLevel));
-                        stack.push_back(WriterContext(key_str, NULL, false, nextLevel));
+                        Py_ssize_t l;
+                        char* key_str = PyUnicode_AsUTF8AndSize(key, &l);
+                        stack.push_back(WriterContext(NULL, 0, item, false, nextLevel));
+                        stack.push_back(WriterContext(key_str, l, NULL, false, nextLevel));
                     }
                     else if (!skipKeys) {
                         PyErr_SetString(PyExc_TypeError, "keys must be a string");
@@ -1031,8 +1044,9 @@ rapidjson_dumps_internal(
 
                 while (PyDict_Next(object, &pos, &key, &item)) {
                     if (PyUnicode_Check(key)) {
-                        char* key_str = PyUnicode_AsUTF8(key);
-                        items.push_back(DictItem(key_str, item));
+                        Py_ssize_t l;
+                        char* key_str = PyUnicode_AsUTF8AndSize(key, &l);
+                        items.push_back(DictItem(key_str, l, item));
                     }
                     else if (!skipKeys) {
                         PyErr_SetString(PyExc_TypeError, "keys must be a string");
@@ -1044,8 +1058,8 @@ rapidjson_dumps_internal(
 
                 std::vector<DictItem>::const_iterator iter = items.begin();
                 for (; iter != items.end(); ++iter) {
-                    stack.push_back(WriterContext(NULL, iter->item, false, nextLevel));
-                    stack.push_back(WriterContext(iter->key_str, NULL, false, nextLevel));
+                    stack.push_back(WriterContext(NULL, 0, iter->item, false, nextLevel));
+                    stack.push_back(WriterContext(iter->key_str, iter->key_size, NULL, false, nextLevel));
                 }
             }
         }
@@ -1060,8 +1074,7 @@ rapidjson_dumps_internal(
             memset(isoformat, 0, ISOFORMAT_LEN);
 
             const int TIMEZONE_LEN = 10;
-            char timezone[TIMEZONE_LEN];
-            memset(timezone, 0, TIMEZONE_LEN);
+            char timeZone[TIMEZONE_LEN] = { 0 };
 
             if (datetimeMode != DATETIME_MODE_ISO8601_IGNORE_TZ
                 && PyObject_HasAttrString(object, "utcoffset")) {
@@ -1080,7 +1093,7 @@ rapidjson_dumps_internal(
                         }
 
                         dtObject = asUTC;
-                        strcpy(timezone, "+00:00");
+                        strcpy(timeZone, "+00:00");
                     } else {
                         PyObject* daysObj = PyObject_GetAttrString(utcOffset, "days");
                         PyObject* secondsObj = PyObject_GetAttrString(utcOffset, "seconds");
@@ -1100,7 +1113,7 @@ rapidjson_dumps_internal(
                             int tz_hour = total_seconds / 3600;
                             int tz_min = (total_seconds % 3600) / 60;
 
-                            snprintf(timezone, TIMEZONE_LEN-1, "%c%02d:%02d", sign, tz_hour, tz_min);
+                            snprintf(timeZone, TIMEZONE_LEN-1, "%c%02d:%02d", sign, tz_hour, tz_min);
                         }
 
                         Py_XDECREF(daysObj);
@@ -1124,14 +1137,14 @@ rapidjson_dumps_internal(
                              "%04d-%02d-%02dT%02d:%02d:%02d.%06d%s",
                              year, month, day,
                              hour, min, sec, microsec,
-                             timezone);
+                             timeZone);
                 } else {
                     snprintf(isoformat,
                              ISOFORMAT_LEN-1,
                              "%04d-%02d-%02dT%02d:%02d:%02d%s",
                              year, month, day,
                              hour, min, sec,
-                             timezone);
+                             timeZone);
                 }
             } else {
                 hour = PyDateTime_TIME_GET_HOUR(dtObject);
@@ -1143,13 +1156,13 @@ rapidjson_dumps_internal(
                              ISOFORMAT_LEN-1,
                              "%02d:%02d:%02d.%06d%s",
                              hour, min, sec, microsec,
-                             timezone);
+                             timeZone);
                 } else {
                     snprintf(isoformat,
                              ISOFORMAT_LEN-1,
                              "%02d:%02d:%02d%s",
                              hour, min, sec,
-                             timezone);
+                             timeZone);
                 }
             }
             Py_XDECREF(asUTC);
@@ -1178,8 +1191,8 @@ rapidjson_dumps_internal(
                 goto error;
 
             // Decref the return value once it's done being dumped to a string.
-            stack.push_back(WriterContext(NULL, NULL, false, currentLevel, retval));
-            stack.push_back(WriterContext(NULL, retval, false, currentLevel));
+            stack.push_back(WriterContext(NULL, 0, NULL, false, currentLevel, retval));
+            stack.push_back(WriterContext(NULL, 0, retval, false, currentLevel));
         }
         else if (defaultFn) {
             PyObject* retval = PyObject_CallFunctionObjArgs(defaultFn, object, NULL);
@@ -1187,8 +1200,8 @@ rapidjson_dumps_internal(
                 goto error;
 
             // Decref the return value once it's done being dumped to a string.
-            stack.push_back(WriterContext(NULL, NULL, false, currentLevel, retval));
-            stack.push_back(WriterContext(NULL, retval, false, currentLevel));
+            stack.push_back(WriterContext(NULL, 0, NULL, false, currentLevel, retval));
+            stack.push_back(WriterContext(NULL, 0, retval, false, currentLevel));
         }
         else {
             PyObject* repr = PyObject_Repr(object);
@@ -1289,18 +1302,30 @@ rapidjson_dumps(PyObject* self, PyObject* args, PyObject* kwargs)
         }
     }
 
-    if (datetimeModeObj && PyLong_Check(datetimeModeObj)) {
-        datetimeMode = (DatetimeMode) PyLong_AsLong(datetimeModeObj);
-        if (datetimeMode < DATETIME_MODE_NONE || datetimeMode > DATETIME_MODE_ISO8601_UTC) {
-            PyErr_SetString(PyExc_ValueError, "Invalid date_time");
+    if (datetimeModeObj) {
+        if (PyLong_Check(datetimeModeObj)) {
+            datetimeMode = (DatetimeMode) PyLong_AsLong(datetimeModeObj);
+            if (datetimeMode < DATETIME_MODE_NONE || datetimeMode > DATETIME_MODE_ISO8601_UTC) {
+                PyErr_SetString(PyExc_ValueError, "Invalid datetime_mode");
+                return NULL;
+            }
+        }
+        else {
+            PyErr_SetString(PyExc_TypeError, "datetime_mode must be an integer value");
             return NULL;
         }
     }
 
-    if (uuidModeObj && PyLong_Check(uuidModeObj)) {
-        uuidMode = (UuidMode) PyLong_AsLong(uuidModeObj);
-        if (uuidMode < UUID_MODE_NONE || uuidMode > UUID_MODE_HEX) {
-            PyErr_SetString(PyExc_ValueError, "Invalid uuid_time");
+    if (uuidModeObj) {
+        if (PyLong_Check(uuidModeObj)) {
+            uuidMode = (UuidMode) PyLong_AsLong(uuidModeObj);
+            if (uuidMode < UUID_MODE_NONE || uuidMode > UUID_MODE_HEX) {
+                PyErr_SetString(PyExc_ValueError, "Invalid uuid_mode");
+                return NULL;
+            }
+        }
+        else {
+            PyErr_SetString(PyExc_TypeError, "uuid_mode must be an integer value");
             return NULL;
         }
     }
@@ -1429,5 +1454,7 @@ PyInit_rapidjson()
         "__author__",
         PYTHON_RAPIDJSON_AUTHOR " <" PYTHON_RAPIDJSON_AUTHOR_EMAIL ">");
 
+    PyModule_AddStringConstant(module, "__rapidjson_version__", RAPIDJSON_VERSION_STRING);
+
     return module;
 }
